@@ -3,6 +3,12 @@ from ipaddress import ip_address
 from tldextract import TLDExtract
 from urllib.parse import urlparse
 
+NO_SCHEME = 'no_scheme'
+HTTP = 'http'
+HTTPS = 'https'
+WS = 'ws'
+WSS = 'wss'
+
 
 def _load_and_update_extractor(function):
     @wraps(function)
@@ -30,6 +36,29 @@ def is_ip_address(hostname):
         return False
 
 
+def _adapt_url_for_port_and_scheme(url, extractor):
+    # To handle the case where we have no scheme, but we have a port
+    # we have the following heuristic. Does scheme have a . in it
+    # which is stdlib behavior when not recognizing a netloc due to
+    # lack of //. If TLDExtract, can find a suffix in the _scheme
+    # then it's probably a domain without an http.
+
+    purl = urlparse(url)
+    _scheme = purl.scheme
+
+    if '.' in str(_scheme):
+        # From the docs: "urlparse recognizes a netloc only
+        # if it is properly introduced by ‘//’". So we
+        # prepend to get results we expect.
+        if extractor(_scheme).suffix != '' or is_ip_address(_scheme):
+            url = '//{url}'.format(url=url)
+    elif url == purl.path:
+        # this is the case where the url has no scheme
+        # and we are trying to access the root. Ex: localhost:5000
+        url = '//{url}/'.format(url=url)
+    return url
+
+
 @_load_and_update_extractor
 def _get_tld_extract(url, **kwargs):
     extractor = kwargs.get('extractor')
@@ -39,18 +68,23 @@ def _get_tld_extract(url, **kwargs):
             "`extractor` keyword argument.")
 
     scheme = kwargs.get('scheme', True)
-    drop_non_http = kwargs.get('drop_non_http', True)
+    path = kwargs.get('path', True)
+    return_unparsed = kwargs.get('return_unparsed', False)
     use_netloc = kwargs.get('use_netloc', True)
-    stripped = get_stripped_url(
+    scheme_default = kwargs.get('scheme_default', HTTP)
+    stemmed = stem_url(
             url,
+            return_unparsed=return_unparsed,
+            scheme_default=scheme_default,
             scheme=scheme,
-            drop_non_http=drop_non_http,
+            path=path,
             use_netloc=use_netloc,
+            extractor=extractor,
     )
-    return extractor(stripped)
+    return extractor(stemmed)
 
 
-def get_ps_plus_1(url, **kwargs):
+def get_etld1(url, **kwargs):
     """
     Returns the eTLD+1 (aka PS+1) of the url.
 
@@ -62,8 +96,8 @@ def get_ps_plus_1(url, **kwargs):
         An (optional) tldextract::TLDExtract instance can be passed with
         keyword `extractor`, otherwise we create and update one automatically.
     kwargs:
-        The method preprocesses the url with ``get_stripped_url`` before
-        extracting the domain. You can pass in ``get_stripped_url`` parameters
+        The method preprocesses the url with ``stem_url`` before
+        extracting the domain. You can pass in ``stem_url`` parameters
         if you wish to change the behavior in some specific way.
 
     Returns
@@ -80,6 +114,11 @@ def get_ps_plus_1(url, **kwargs):
         return f'{parsed.domain}.{parsed.suffix}'
 
 
+def get_ps_plus_1(url, **kwargs):
+    """An alias for ``get_etld1``."""
+    return get_etld1(url, **kwargs)
+
+
 @_load_and_update_extractor
 def hostname_subparts(url, include_ps=False, **kwargs):
     """
@@ -97,7 +136,7 @@ def hostname_subparts(url, include_ps=False, **kwargs):
         * ``["a.b.c.d.com", "b.c.d.com", "c.d.com", "d.com"]`` if ``include_ps == False``
         * ``["a.b.c.d.com", "b.c.d.com", "c.d.com", "d.com", "com"]`` if ``include_ps == True``
     kwargs:
-        Additionally all kwargs for get_ps_plus_1, can be passed to this method.
+        Additionally all kwargs for get_etld1, can be passed to this method.
 
     Returns
     -------
@@ -105,16 +144,16 @@ def hostname_subparts(url, include_ps=False, **kwargs):
         List of slices of of a url's hostname down to the eTLD+1 / PS+1.
     """
     ext = _get_tld_extract(url, **kwargs)
-    ps_plus_1 = get_ps_plus_1(url, **kwargs)
+    etld1 = get_etld1(url, **kwargs)
 
     # If an IP address, just return a single item list with the IP
     if is_ip_address(ext.domain):
         return [ext.domain]
 
-    # We expect all ps_plus_1s to have at least one '.'
+    # We expect all eTLD+1s to have at least one '.'
     # If they don't, the url was likely malformed, so we'll just
     # return an empty list
-    if '.' not in ps_plus_1:
+    if '.' not in etld1:
         return []
 
     # Build a string of the URL except the suffix
@@ -141,7 +180,15 @@ def hostname_subparts(url, include_ps=False, **kwargs):
 
 
 @_load_and_update_extractor
-def get_stripped_url(url, scheme=False, drop_non_http=False, use_netloc=True, extractor=None):
+def stem_url(
+        url,
+        return_unparsed=True,
+        scheme_default=HTTP,
+        parse_ws=True,
+        scheme=False,
+        path=True,
+        use_netloc=True,
+        extractor=None):
     """
     Returns a url stripped to just the beginning and end.
 
@@ -153,30 +200,40 @@ def get_stripped_url(url, scheme=False, drop_non_http=False, use_netloc=True, ex
     `urllib.parse.urlparse
     <https://docs.python.org/3.8/library/urllib.parse.html>`_.
 
-    Empty scheme e.g. ``my.domain.cloudfront.net``
-    are assumed to be http schemes.
+    A url is parsed if it has a qualifying scheme. The qualifying schemes are
+    ``http``, ``https``, ``ws`` and ``wss``. Websocket schemes can be omitted using
+    the ``parse_ws`` parameter. Additionally, the ``scheme_default`` parameter
+    provides a scheme where the url doesn't contain one. The default is ``http``
+    and so urls without a scheme will, by default, be considered as http and therfore
+    parsed.
 
-    If a URL has a port but no scheme, urlparse determines the scheme to
-    be the hostname and we do not handle this special case. In this case,
-    the url will be treated as a non-http scheme and the return value will
-    be determined by the ``drop_non_http`` setting.
+    What is returned for unparsed urls is determined by the ``return_unparsed``
+    parameter.
 
     Parameters
     ----------
     url : string
         The URL to be parsed
+    return_unparsed : boolean, optional
+        Action to take if scheme is not parsed e.g. ``file:`` or ``about:blank``.
+        If ``False``, the result for non parsed urls will be an empty string
+        If ``True``, the result will be the original url, e.g.
+        ``about:blank`` -> ``about:blank`` even if ``scheme=False``.
+        See method description to understand whether a URL is parsed or not.
+        Default is ``True``.
+    scheme_default : string, optional
+        This parameter is passed to scheme parameter of `urllib.parse.urlparse`. This
+        causes urls without a scheme to return the scheme default.
+        Default is ``http``.
+    parse_ws : boolean, optional
+        If ``True``, then ``ws`` and ``wss`` urls are parsed.
+        Default is ``True``.
     scheme : boolean, optional
-        If ``True``, scheme will be prepended in returned result.
-        Default is ``False``.
-    drop_non_http : boolean, optional
-        Action to take if scheme is not
-        ``http`` or ``https`` e.g. ``file:`` or ``about:blank``.
-        If ``True``, the result for non http urls will be an empty string
-        If ``False``, the result for non http urls will be the original url,
-        not further processed e.g. ``about:blank`` -> ``about:blank`` even
-        if ``scheme=False``. The result for http urls will be the stripped
-        url with or without the scheme as per scheme param.
+        If ``True``, scheme will be prepended in parsed result.
         Default is ``False``.
+    path : boolean, optional
+        If ``True``, path will be included in parsed result.
+        Default is ``True``.
     use_netloc : boolean, optional
         If ``True`` urlparse's netloc will be used.
         If ``False`` urlparse's host will be returned. Using netloc means
@@ -192,40 +249,31 @@ def get_stripped_url(url, scheme=False, drop_non_http=False, use_netloc=True, ex
         Returns a url stripped to (scheme)?+(netloc|hostname)+(path)?.
         Returns empty string if appropriate.
     """
-    purl = urlparse(url)
+    url = _adapt_url_for_port_and_scheme(url, extractor)
+
+    purl = urlparse(url, scheme=scheme_default)
     _scheme = purl.scheme
 
-    # To handle the case where we have no scheme, but we have a port
-    # we have the following heuristic. Does scheme have a . in it
-    # which is stdlib behavior when not recognizing a netloc due to
-    # lack of //. If TLDExtract, can find a suffix in the _scheme
-    # then it's probably a domain without an http.
-    if '.' in _scheme:
-        # From the docs: "urlparse recognizes a netloc only
-        # if it is properly introduced by ‘//’". So we
-        # prepend to get results we expect.
-        if extractor(_scheme).suffix != '' or is_ip_address(_scheme):
-            url = '//{url}'.format(url=url)
-
-    purl = urlparse(url)
-    _scheme = purl.scheme
-
-    # Handle non http schemes
-    if _scheme not in ['http', 'https', '']:
-        if drop_non_http is True:
-            return ''
-        else:
+    # Will we parse
+    schemes_to_parse = [HTTP, HTTPS]
+    if parse_ws is True:
+        schemes_to_parse += [WS, WSS]
+    if _scheme not in schemes_to_parse:
+        if return_unparsed is True:
             return url
+        return ''
 
-    purl = urlparse(url)
     scheme_out = ''
     loc_out = ''
-    path_out = purl.path
+    path_out = ''
 
     if scheme is True:
-        if _scheme in ['http', 'https']:
+        if _scheme in schemes_to_parse:
             scheme_out = '{scheme}://'.format(scheme=_scheme)
 
+    if path is True:
+        path_out = purl.path
+
     if use_netloc is True:
         loc_out = purl.netloc
     else:
@@ -236,3 +284,57 @@ def get_stripped_url(url, scheme=False, drop_non_http=False, use_netloc=True, ex
         loc_out=loc_out,
         path_out=path_out,
     )
+
+
+def get_stripped_url(url, **kwargs):
+    """Alias for ``stem_url``."""
+    return stem_url(url, **kwargs)
+
+
+def get_scheme(url, no_scheme=NO_SCHEME):
+    """
+    Given an url, extract from it the scheme.
+
+    Parameters
+    ----------
+    url: string
+        The URL from where we want to get the scheme
+    no_scheme: any
+        The value to use if no scheme is detected.
+        Default is ``no_scheme``
+
+    Returns
+    -------
+    string
+        Returns the scheme with a default of 'blank' if no schema is provided
+    """
+
+    scheme = urlparse(url).scheme
+
+    if scheme:
+        return scheme
+    else:
+        return no_scheme
+
+
+@_load_and_update_extractor
+def get_port(url, extractor=None):
+    """
+    Given an url, extract from it port if present.
+
+    Parameters
+    ----------
+    url: string
+        The URL from where we want to get the scheme
+    extractor : tldextract::TLDExtract, optional
+        An (optional) tldextract::TLDExtract instance can be passed with
+        keyword `extractor`, otherwise we create and update one automatically.
+
+    Returns
+    ----------
+    int
+        Returns port in the url. If port not found, returns ``None``.
+    """
+
+    url = _adapt_url_for_port_and_scheme(url, extractor)
+    return urlparse(url).port
